#!/usr/bin/env node
'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.parseHeaders = parseHeaders;
exports.processHeaders = processHeaders;
exports.createTOC = createTOC;

var _commander = require('commander');

var _commander2 = _interopRequireDefault(_commander);

var _superagent = require('superagent');

var _superagent2 = _interopRequireDefault(_superagent);

var _opn = require('opn');

var _opn2 = _interopRequireDefault(_opn);

var _chalk = require('chalk');

var _chalk2 = _interopRequireDefault(_chalk);

var _package = require('./../package.json');

var _package2 = _interopRequireDefault(_package);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function parseHeaders(readMeText) {
  var headers = [];
  var lines = readMeText.split('\n');
  var isCodeBlock = false;

  for (var i = 0; i < lines.length; i++) {
    // Extract contents of markdown links
    if (/\[([^\]]+)\][^)]+\)/g.test(lines[i])) {
      lines[i] = lines[i].replace(/\[([^\]]+)\][^)]+\)/g, '$1');
    }

    // Toggle whether or not text is within a code block
    if (lines[i].includes('```')) isCodeBlock = !isCodeBlock;

    if (!isCodeBlock) {
      // Identify alternate H1 & H2 headers using underline-based styles
      if (i !== lines.length - 1) {
        if (/^=+$/.test(lines[i + 1])) {
          headers.push({ depth: 1, text: lines[i] });
        } else if (/^-+$/.test(lines[i + 1])) {
          headers.push({ depth: 2, text: lines[i] });
        }
      }

      // Identify headers using the '#' character
      if (/^#{1,6}.+$/.test(lines[i])) {
        var headerDepth = lines[i].lastIndexOf('#', 5) + 1;
        var headerText = lines[i].substring(headerDepth);

        headers.push({ depth: headerDepth, text: headerText });
      }
    }
  }

  return headers;
}

function processHeaders(headers, depth) {
  var processedHeaders = [];
  var usedAnchors = {};

  /*
    Algorithm for creating anchors:
    1. Strip whitespace from both ends
    2. Lowercase
    3. Remove any character that is not a letter, number, space, or hyphen
    4. Change any space to a hyphen
    5. If anchor is not unique within the document, concat '-1', '-2', etc.
  */
  var createAnchor = function createAnchor(text) {
    var anchor = text.trim().toLowerCase().replace(/[^\w\s-]/g, '').replace(/\s/g, '-');

    if (usedAnchors[anchor]) {
      usedAnchors[anchor]++;
      anchor = anchor + '-' + usedAnchors[anchor];
    } else {
      usedAnchors[anchor] = 1;
    }

    return anchor;
  };

  for (var i = 0; i < headers.length; i++) {
    if (headers[i].depth <= depth) {
      processedHeaders.push({
        depth: headers[i].depth,
        text: headers[i].text.trim(),
        anchor: createAnchor(headers[i].text)
      });
    }
  }

  return processedHeaders;
}

function createTOC(headers) {
  var TOC = '';

  for (var i = 0; i < headers.length; i++) {
    TOC = TOC.concat(' '.repeat(headers[i].depth * 2 - 1) + '* [' + headers[i].text + '](#' + headers[i].anchor + ')\n');
  }

  return TOC;
}

function handleError(err, res) {
  var errorMessage = void 0;
  if (res && res.status === 404) {
    errorMessage = '404: Markdown file for the specified repository not found.';
  } else if (err) {
    errorMessage = err;
  } else {
    errorMessage = res.text;
  }

  console.error(_chalk2.default.red(errorMessage));
  process.exit(1);
}

function generateReadMeTOC(user, repository, file) {
  _superagent2.default.get('https://raw.githubusercontent.com/' + user + '/' + repository + '/master/' + file).set('Accept', 'text/plain').end(function (err, res) {
    if (res && res.ok) {
      if (_commander2.default.open) {
        (0, _opn2.default)('https://github.com/' + user + '/' + repository + '/blob/master/' + file);
        process.exit(0);
      }

      var headers = parseHeaders(res.text);

      var depth = _commander2.default.depth ? Number(_commander2.default.depth) : 6;
      headers = processHeaders(headers, depth);

      var TOC = createTOC(headers);
      console.log(_chalk2.default.blue(TOC));

      process.exit(0);
    } else {
      handleError(err, res);
    }
  });
}

_commander2.default.version(_package2.default.version).usage('[options] <user> <repository> <file>').description('A command-line utility that generates Tables of Contents for Github READMEs.').option('-d, --depth <number>', 'specifiy the maximum header depth (1 - 6) of the toc').option('-o, --open', 'open the readme in browser').parse(process.argv);

if (_commander2.default.args.length !== 2 && _commander2.default.args.length !== 3) {
  _commander2.default.help();
} else {
  var user = _commander2.default.args[0];
  var repository = _commander2.default.args[1];
  var file = _commander2.default.args[2] ? _commander2.default.args[2] : 'README.md';

  generateReadMeTOC(user, repository, file);
}